@@ -10,8 +10,10 @@ package Test::Filesys::POSIX::Error;
 use strict;
 use warnings;
 
-use base 'Test::Builder::Module';
 use Errno;
+use Try::Tiny;
+
+use base 'Test::Builder::Module';
 
 our @EXPORT = qw(throws_errno_ok);
 
@@ -20,7 +22,13 @@ my %cache;
 
 BEGIN {
     #
-    # Store a number => symbolic name mapping
+    # First, store a basic table of number => number mappings as a fallback
+    # for when symbols cannot be pulled reliably from the Errno stash.
+    #
+    %cache = map { $_ => $_ } 0 .. 255;
+
+    #
+    # Next, attempt to store a number => symbolic name mapping.
     #
     foreach my $name ( @Errno::EXPORT_OK, @{ $Errno::EXPORT_TAGS{'POSIX'} } ) {
         my $sv = $Errno::{$name};
@@ -40,11 +48,14 @@ sub throws_errno_ok (&$$) {
 
     local $@;
 
-    eval { $sub->(); };
+    try {
+        $sub->();
+        $pass = 0;
+    }
+    catch {};
 
     my $found = int $!;
 
-    $pass = 0 unless $@;
     $pass = 0 unless $found == $errno;
 
     $builder->ok( $pass, $message );
